// Description: Defines used in nvcdis.c

// History:
// 8 Apr 1998 - Created
// 9 Apr 1998 - Version 1.0 completed
// Many mods by David Tucker....
// v???? August-October 2015 - Comment insertion, MV810ASM-like output (MH)

// Authors:
// Bob VanderClay (vandercl@umich.edu)
// David Tucker (dbt@dana.ucc.nau.edu)
// Amos Bieler (abieler@gmx.net)
// Matej Horvat (matej.horvat@guest.arnes.si)

#ifndef V810_DS2_H_
#define V810_DS2_H_

#include <stdint.h>

typedef uint32_t word;
typedef uint16_t hword;
typedef uint8_t byte;

///////////////////////////////////////////////////////////////////
// Address Modes
#define AM_I	1
#define AM_IIa	21
#define AM_IIb	22		// Unsigned (hex) immediate
#define AM_IIc	23		// Bit String Instructions
#define AM_IId	24		// No arguments
#define AM_IIe	25		// One (zero-extended) argument
#define AM_III	3
#define AM_IV	4
#define AM_V	51
#define AM_Vb	52		// Sign-extended
#define AM_VIa	61		// Mode6 form1
#define AM_VIb	62		// Mode6 form2
#define AM_VII	7		// Extended (floating-point and NVC custom) Instructions
#define AM_RFU	255		// RFU stands for "Reserved for Future Use"


///////////////////////////////////////////////////////////////////
// Instruction and Address Mode Table

typedef struct op {
	int addr_mode;		// Addressing mode
	char * opname;		// opcode name (string)
} operation;

static operation optable[80] = {
	{ AM_I,		"mov"	},	// 0x00
	{ AM_I,		"add"	},	// 0x01
	{ AM_I,		"sub"	},	// 0x02
	{ AM_I,		"cmp"	},	// 0x03
	{ AM_I,		"shl"	},	// 0x04
	{ AM_I,		"shr"	},	// 0x05
	{ AM_I,		"jmp"	},	// 0x06
	{ AM_I,		"sar"	},	// 0x07
	{ AM_I,		"mul"	},	// 0x08
	{ AM_I,		"div"	},	// 0x09
	{ AM_I,		"mulu"	},	// 0x0A
	{ AM_I,		"divu"	},	// 0x0B
	{ AM_I,		"or"	},	// 0x0C
	{ AM_I,		"and"	},	// 0x0D
	{ AM_I,		"xor"	},	// 0x0E
	{ AM_I,		"not"	},	// 0x0F

	{ AM_IIa,	"mov"	},	// 0x10 // Immediate
	{ AM_IIa,	"add"	},	// 0x11
	{ AM_IIb,	"setf"	},	// 0x12
	{ AM_IIa,	"cmp"	},	// 0x13
	{ AM_IIb,	"shl"	},	// 0x14
	{ AM_IIb,	"shr"	},	// 0x15
	{ AM_IId,	"cli"	},	// 0x16 // Clear Interrupt Disable (NVC/VB Specific)
	{ AM_IIb,	"sar"	},	// 0x17
	{ AM_IIe,	"trap"	},	// 0x18

	{ AM_IId,	"reti"	},	// 0x19
	{ AM_IId,	"halt"	},	// 0x1A

	{ AM_RFU,	""		},	// 0x1B // Unknown
	{ AM_IIa,	"ldsr"	},	// 0x1C
	{ AM_IIa,	"stsr"	},	// 0x1D
	{ AM_IId,	"sei"	},	// 0x1E // Set Interrupt Disable (NVC/VB Specific)
	{ AM_IIc,	"BSTR"	},	// 0x1F // Special Bit String Instructions

	{ AM_RFU,	""		},	// 0x20	// This is a kludge to accommodate the
	{ AM_RFU,	""		},	// 0x21 // conditional-branch instructions.
	{ AM_RFU,	""		},	// 0x22
	{ AM_RFU,	""		},	// 0x23
	{ AM_RFU,	""		},	// 0x24
	{ AM_RFU,	""		},	// 0x25
	{ AM_RFU,	""		},	// 0x26
	{ AM_RFU,	""		},	// 0x27

	{ AM_Vb,	"movea"	},	// 0x28
	{ AM_Vb,	"addi"	},	// 0x29
	{ AM_IV,	"jr"	},	// 0x2A
	{ AM_IV,	"jal"	},	// 0x2B
	{ AM_V,		"ori"	},	// 0x2C
	{ AM_V,		"andi"	},	// 0x2D
	{ AM_V,		"xori"	},	// 0x2E
	{ AM_V,		"movhi"	},	// 0x2F
	{ AM_VIa,	"ld.b"	},	// 0x30
	{ AM_VIa,	"ld.h"	},	// 0x31
	{ AM_RFU,	""		},	// 0x32
	{ AM_VIa,	"ld.w"	},	// 0x33
	{ AM_VIb,	"st.b"	},	// 0x34
	{ AM_VIb,	"st.h"	},	// 0x35
	{ AM_RFU,	""		},	// 0x36
	{ AM_VIb,	"st.w"	},	// 0x37
	{ AM_VIa,	"in.b"	},	// 0x38
	{ AM_VIa,	"in.h"	},	// 0x39
	{ AM_VIa,	"caxi"	},	// 0x3A
	{ AM_VIa,	"in.w"	},	// 0x3B
	{ AM_VIb,	"out.b"	},	// 0x3C
	{ AM_VIb,	"out.h"	},	// 0x3D
	{ AM_VII,	"EXT"	},	// 0x3E  Extended instruction
	{ AM_VIb,	"out.w"	},	// 0x3F

	{ AM_III,	"bv"	},	// 0x40
	{ AM_III,	"bl"	},	// 0x41 (BC)
	{ AM_III,	"be"	},	// 0x42 (BZ)
	{ AM_III,	"bnh"	},	// 0x43
	{ AM_III,	"bn"	},	// 0x44
	{ AM_III,	"br"	},	// 0x45
	{ AM_III,	"blt"	},	// 0x46
	{ AM_III,	"ble"	},	// 0x47
	{ AM_III,	"bnv"	},	// 0x48
	{ AM_III,	"bnl"	},	// 0x49 (BNC)
	{ AM_III,	"bne"	},	// 0x4A (BNZ)
	{ AM_III,	"bh"	},	// 0x4B
	{ AM_III,	"bp"	},	// 0x4C
	{ AM_III,	"nop"	},	// 0x4D
	{ AM_III,	"bge"	},	// 0x4E
	{ AM_III,	"bgt"	},	// 0x4F
};

//  Bit String Subopcodes
static char * bssuboptable[16] = {
	"SCH0BSU",				// 0x00
	"SCH0BSD",				// 0x01
	"SCH1BSU",				// 0x02
	"SCH1BSD",				// 0x03
	"RFU_B04",				// 0x04
	"RFU_B05",				// 0x05
	"RFU_B06",				// 0x06
	"RFU_B07",				// 0x07
	"ORBSU",				// 0x08
	"ANDBSU",				// 0x09
	"XORBSU",				// 0x0A
	"MOVBSU",				// 0x0B
	"ORNBSU",				// 0x0C
	"ANDNBSU",				// 0x0D
	"XORNBSU",				// 0x0E
	"NOTBSU",				// 0x0F
};

//  Floating Point Subopcodes
static char * fpsuboptable[16] = {
	"cmpf.s",				// 0x00
	"RFU_F01",				// 0x01
	"cvt.ws",				// 0x02
	"cvt.sw",				// 0x03
	"addf.s",				// 0x04
	"subf.s",				// 0x05
	"mulf.s",				// 0x06
	"divf.s",				// 0x07
	"xb",					// 0x08	// Exchange Byte (NVC/VB Specific)
	"xh",					// 0x09	// Exchange Half Word (NVC/VB Specific)
	"rev",					// 0x0A // Reverse Bits in Word (NVC/VB Specific)
	"trnc.sw",				// 0x0B
	"mpyhw",				// 0x0C // Multiply Half Word (NVC/VB Specific)
	"RFU_F0D",				// 0x0D
	"RFU_F0E",				// 0x0E
	"RFU_F0F",				// 0x0F
};

/*/////////////////////////////////////////////////////////////////*/
#define         JMP				0x06
#define         LDSR			0x1C
#define         STSR			0x1D

#endif //DEFINE_H
